
* data - 999 used instead of . to indicate MISSING;

data old;
input a b c d e;
datalines;
10 20 30 999 50
50 40 999 20 10
999 1 1 1 999
;
run;

title 'ORIGINAL NUMERIC DATA - MISSING = 999';
proc print data=old noobs;
run;

* write an IF statement to examine each variable - not so tedious if
only 5 variables what about 10, 20, etc.;

data new;
set old;
if a = 999 then a = .;
if b = 999 then b = .;
if c = 999 then c = .;
if d = 999 then d = .;
if e = 999 then e = .;
run;

title 'NUMERIC 999 FIXED USING A DATA STEP WITHOUT AN ARRAY';
proc print data=new noobs;
run;

* use an ARRAY - no way to learn about ARRAYS
without also learning about DO LOOPS;

data new;
array x(5) a b c d e;
set old;
do i=1 to 5;
   if x(i) eq 999 then x(i) = .;
end;
drop i;
run;

title 'NUMERIC 999 FIXED USING AN ARRAY';
proc print data=new noobs;
run;

* change names of variables to x1-x5;

data old;
input x1-x5;
datalines;
10 20 30 999 50
50 40 999 20 10
999 1 1 1 999
;
run;

title 'ORIGINAL NUMERIC DATA - MISSING = 999 ... COMMON PREFIX, NUMERIC SUFFIX';
proc print data=old noobs;
run;

data new;
array x(5);
set old;
do i=1 to 5;
   if x(i) eq 999 then x(i) = .;
end;
drop i;
run;

title 'NUMERIC 999 FIXED USING AN ARRAY';
proc print data=new noobs;
run;


* try a shortcut - refer to all numeric values with _numeric_;
* will not work - location of array statement sometimes matters
what variables are numeric - not known until after the SET statement;

data old;
input a b c d e;
datalines;
10 20 30 999 50
50 40 999 20 10
999 1 1 1 999
;
run;

title 'BACK TO ORIGINAL NUMERIC DATA - MISSING = 999';
proc print data=old noobs;
run;

data new;
array x(5) _numeric_;
set old;
do i=1 to 5;
   if x(i) eq 999 then x(i) = .;
end;
drop i;
run;

* move the ARRAY statement AFTER the SET;

data new;
set old;
array x(5) _numeric_;
do i=1 to 5;
   if x(i) eq 999 then x(i) = .;
end;
drop i;
run;

title 'NUMERIC 999 FIXED USING AN ARRAY WITH _NUMERIC_';
proc print data=new noobs;
run;

* another shortcut;

data old;
input xa xb xc xd xe;
datalines;
10 20 30 999 50
50 40 999 20 10
999 1 1 1 999
;
run;

title 'ORIGINAL NUMERIC DATA - MISSING = 999 ... COMMON PREFIX';
proc print data=old noobs;
run;

data new;
set old;
array x(5) x: ;
do i=1 to 5;
   if x(i) eq 999 then x(i) = .;
end;
drop i;
run;

title 'NUMERIC 999 FIXED USING AN ARRAY WITH _NUMERIC_';
proc print data=new noobs;
run;

* same example, more numeric variables;
data old;
input a b c d e f g h i j k l m n o p q r s t;
datalines;
999 20  30 999  50  10  20 30  999 50  10 20 30   999  50  10  20 30  999 999
999 40   0  20  10  50  40 999 20  10  50 40 999  20   10  50  40 999  20  10
999  1   1  1  999 999   1 1    1 999  87  1   1  999 999 999   1   1  1  999
;
run;

title 'ORIGINAL NUMERIC DATA - MISSING = 999';
proc print data=old noobs;
run;

* use an ASTERISK in the array statement;
* let SAS count the number of numeric variables;
* use the DIM function to set the number of iterations for the DO LOOP
* QUESTION - why not use I in the LOOP as in the previous example? 
*            why use _n_?
;

data new;
set old;
array x(*) _numeric_;
do _n_=1 to dim(x);
   if x(_n_) eq 999 then x(_n_) = .;
end;
run;

title 'NUMERIC 999 FIXED USING AN ARRAY WITH _NUMERIC_';
proc print data=new noobs;
run;

* determine what variable was fixed - CALL VNAME;

data new (drop=var_name) fixed (keep=var_name);
length var_name $32;
set old;
array x(*) _numeric_;
do _n_ = 1 to dim(x);
   if x(_n_) eq 999 then do; 
      x(_n_) = .;
	  var_name = vname(x(_n_));
*	  call vname (x(_n_),var_name);
	  output fixed;
   end;
end;
output new;
run;

title 'NUMERIC 999 FIXED USING AN ARRAY WITH _NUMERIC_';
proc print data=new noobs;
run;

title 'VARIABLES CHANGED FROM 999 TO MISSING';
proc freq data=fixed;
run;



* three diagnoses entered - n/a used to indicate MISSING (not/available);
data old;
length dx1-dx3 $5;
input dx1-dx3;
datalines;
630 64212 410
6340 78559 n/a
6442 n/a 2875

run;

* character array - arrays assumed to be numeric unless otherwise stated;
data new;
array diag(3) $5 dx1-dx3;
set old;
do i=1 to 3;
   if diag(i) eq 'n/a' then diag(i) = ' ';
end;
drop i;
run;

title 'CHARACTER N/A FIXED USING AN ARRAY';
proc print data=new noobs;
run;

* use _character_;

data new;
set old;
array diag(*) _character_;
do i=1 to dim(diag);
   if diag(i) eq 'n/a' then diag(i) = ' ';
end;
drop i;
run;

title 'CHARACTER N/A FIXED USING AN ARRAY AND _CHARACTER_';
proc print data=new noobs;
run;

* mix of numeric and character data;

data old;
input a b c d e (dx1-dx3) (: $5.);
datalines;
10 20 30 999 50 630 64212 410
50 40 999 20 10 6340 78559 n/a
999 1 1 1 999 6442 n/a 2875
;
run;

* use _numeric_ and _character_;

data new;
set old;
array ch(*) _character_;
array nm(*) _numeric_;

do ijk=1 to dim(ch);
   if ch(ijk) eq 'n/a' then ch(ijk) = ' ';
end;

do ijk=1 to dim(nm);
   if nm(ijk) eq 999 then nm(ijk) = .;
end;

drop ijk;
run;

title 'CHARACTER N/A AND MUMERIC 999 FIXED USING ARRAYS';
proc print data=new noobs;
run;

* multiple arrays;

data convert1;
input ht1-ht3 wt1-wt3;
datalines;
60 66 72 100 150 200
;
run;

title 'DATA SET CONVERT1 - NO ARRAYS';
proc print data=convert1 noobs;
run;

data convert2 (keep=htcm: wtkg:);
array ht(3);
array htcm(3);
array wt(3);
array wtkg(3);
input ht1-ht3 wt1-wt3;
do k = 1 to 3;
    htcm(k) = 2.54 * ht(k);
    wtkg(k) = wt(k) / 2.2;
end;
datalines;
60 66 72 100 150 200
run;

title 'DATA SET CONVERT2 - ARRAYS';
proc print data=convert2 noobs;
run;

* use the same variable names;

data convert3 (drop=k);
array ht(3);
array wt(3);
input ht1-ht3 wt1-wt3;
do k = 1 to 3;
ht(k) = 2.54 * ht(k);
wt(k) = wt(k) / 2.2;
end;
datalines;
60 66 72 100 150 200
run;

title 'DATA SET CONVERT3 - ARRAYS - SAME VARIABLE NAMES';
proc print data=convert3 noobs;
run;

* temporary arrays, assigning values to array elements;

data passing;
array pass(5) _temporary_ (65 70 65 80 75);
array score(5);

input id $ score(*);

* pass_num = 0;
do i = 1 to 5;
   if score(i) ge pass(i) then pass_num + 1;
*   if score(i) ge pass(i) then pass_num = pass_num + 1;
*   pass_num + (score(i) ge pass(i));
*    pass_num = sum (pass_num,  (score(i) ge pass(i)));
end;

drop i;
datalines;
001 64 69 68 82 74
002 80 80 80 60 80
;
run;

title 'DATA SET PASSING';
proc print data=passing noobs;
var id pass_num score1-score5;
run;


* same methodology, use the CLINICAL data and set risk values
* set at-risk values for chol, sbp, dbp, hr
* use PICTURE formats to mark at-risk values;

libname x 'k:\sasclass\sasdata';

proc print data=x.clinical;
format _all_;
run;

data risk;
array risk(4) _temporary_ (350 150 100 80);
array x(4) chol sbp dbp hrate;
set x.clinical;
* nrisk = 0;
do j = 1 to 4;
*   if x(j) gt risk(j) then  nrisk+1;
*  nrisk + (x(j) gt risk(j)) ;
  nrisk = sum(nrisk,(x(j) gt risk(j)));
end;
format _all_;
run;

proc format;
picture chol 
350 - high = '099*'
other      = '099 ';
picture sbp
150 - high = '099*'
other      = '099 ';
picture dbp
100 - high = '099*'
other      = '099 ';
picture hrate
80 - high  = '099*'
other      = '099 ';
run;

title 'DATA SET RISK - * INDICATES VALUE AT RISK';
proc print data=risk noobs;
var patient nrisk chol sbp dbp hrate;
format chol chol. sbp sbp. dbp dbp. hrate hrate.;
run;

********************
* TRAFFIC LIGHTING 
********************;

* look for asthma DX in secondary diagnoses;

filename adr saszipam "h:\adrnys02.zip";

data asthma;
* count records - write count to LOG at 50,000 record intervals;
if mod(_n_,50000) eq 0 then put _n_ comma10.;

* file with ZIP archive;
infile adr(adrnys02.txt) lrecl=450 pad obs=100000;

* array of 14 secondary diagnoses;
array odx(14) $5;

* read diagnoses;
input
@065 adx $5.
@071 pdx $5.
@077 (odx1-odx14) ($5. +3)
;

* check admit and primary DX for asthma - expression on left TRUE = 1, FALSE = 0;
admit   = (adx eq : '493');
primary = (pdx eq : '493');

* check all seconday DX for asthma - stop loop when asthma is found;
do j=1 to 14 until (secondary eq 1);
*do j=1 to 14 while (secondary in (. 0));
   secondary = (odx(j) eq : '493');
end;

* if any type of diagnosis is asthma, output observation;
if sum (of admit primary secondary) gt 0;

drop j;
run;

* check observations for type of diagnosis;
proc freq data=asthma;
table admit primary secondary / list;
run;

********************
* CAT FUNCTIONS 
********************;

* rearrange data (array instead of PROC TRANSPOSE);

data many_dx;
infile datalines missover;
length id $2 dx1-dx5 $3;
input id dx1-dx5;
datalines;
01 647 038
02 428 518 416
03 642 674 431 648
04 415 280 997 427 453
05 641 668 648 666 641
06 670 041 728 427 518
07 864 866
08 648 801
09 666 666 286 286
10 659 560 348 507
;
run;

title 'MANY DX PER OBSERVATION';
proc print data=many_dx;
run;

data diags (keep=diag);
set many_dx;
array dx(5);
do j=1 to 5;
   diag=dx(j);
   if diag ne '' then output;
end;
run;

title 'USE AN ARRAY IN A DATA STEP';
proc freq data=diags;
table diag;
run;

* more data ... just change the size of the array;
data many_dx;
infile datalines missover;
length id $2 dx1-dx15 $3;
input id dx1-dx15;	
datalines;
647 038 320 348 V66
428 518 416 710 710 424 666 443
642 674 431 648 584 427 518 427 V46 038 648 663 666 648 535
415 280 997 427 453 626 414 250 218 458
641 668 648 666 641 668 668 669 669 672 666 665 664 648 622
;
run;

data diags (keep=diag);
set many_dx;
array dx(*) dx: ;	
do j=1 to dim(dx);
   diag=dx(j);
   if diag ne ' ' then output;
end;
run;

proc freq data=diags;
table diag;
run;

* transpose multiple variables;

libname o 'k:\epi697\datasets';

proc print data=o.one;
run;

data many;
set o.one;
array s(10) sbp1-sbp10;
array d(10) dbp1-dbp10;
* three different ideas;
do j=1 to 10;
*do j=1 to 10 until(s(j) eq .);
*do j=1 to 10 while(s(j) ne .);
   sbp = s(j);
   dbp = d(j);
   output;
end;
keep id sbp dbp;
run;

proc print data=many;
run;
